#include <string.h>

#include "my_stdio.h"  /* for multi-volume file systems; set up for Mac */
#include "Un*xFiles.h"

int fr ( FILE *fd, char *buf, int cnt );
int decode ( FILE *in, FILE *out );
int doutdec(char *p, FILE *f, int n);
char *index ( char *sp, char c );
int encode( FILE *in, FILE *out);
int eoutdec ( char *p, FILE *f);

int fr ( FILE *fd, char *buf, int cnt ) {
	int c, i;

	for (i=0; i<cnt; i++) {
		c = getc(fd);
		if (c == EOF)
			return(i);
		buf[i] = c;
	}
	return (cnt);
}

/************************************************************************/

/* single character decode */
#define DEC(c)	(((c) - ' ') & 077)

int uudecode ( char *source ) {
	FILE *in, *out;
	int mode, result;
	char dest[31], buf[80];

	if (!(in = my_fopen(source, "r")))
		return ( NOOPIN );

	/* search for header line */
	for (;;) {
		if (fgets(buf, sizeof buf, in) == NULL)
			return ( FORMAT );
		if (strncmp(buf, "begin ", 6) == 0)
			break;
	}
	sscanf(buf, "begin %o %s", &mode, dest);

	/* create output file */
	out = my_fopen(dest, "w");
	if (out == NULL)
		return ( NOOPOUT );

	if ( result = decode (in, out) )
		return ( result );

	if (fgets(buf, sizeof buf, in) == NULL || strcmp(buf, "end\n"))
		return ( DETRANS );

	return ( NOERR );
}

int decode ( FILE *in, FILE *out ) {
	char buf[80];
	char *bp;
	int n, result;

	for (;;) {
		/* for each input line */
		if (fgets(buf, sizeof buf, in) == NULL)
			return ( NOREAD );
		n = DEC(buf[0]);
		if (n <= 0)
			break;

		bp = &buf[1];
		while (n > 0) {
			if ( result = doutdec(bp, out, n) )
				return ( result );
			bp += 4;
			n -= 3;
		}
	}
	return ( NOERR );
}

int doutdec(char *p, FILE *f, int n) {
	int c1, c2, c3;

	c1 = DEC(*p)   << 2 | DEC(p[1]) >> 4;
	c2 = DEC(p[1]) << 4 | DEC(p[2]) >> 2;
	c3 = DEC(p[2]) << 6 | DEC(p[3]);
	if (n >= 1)
		if ( EOF == putc(c1, f) )
			return ( NOWRITE );
	if (n >= 2)
		if ( EOF == putc(c2, f) )
			return ( NOWRITE );
	if (n >= 3)
		if ( EOF == putc(c3, f) )
			return ( NOWRITE );
	return ( NOERR );
}

char *index ( char *sp, char c ) {
	do {
		if (*sp == c)
			return(sp);
	} while (*sp++);
	return(NULL);
}

/**********************************************************************/

/* ENC is the basic 1 character encoding function to make a char printing */
#define ENC(c) (((c) & 077) + ' ')

int uuencode (char *source, char *target) {
	FILE *in, *out;
	int result;

	if ( ( in = my_fopen ( source,  "r" ) ) == NULL )
		return ( NOOPIN );

	if ( ( out = my_fopen ( target, "w" ) ) == NULL )
		return ( NOOPOUT );

	fprintf(out, "begin %o %s\n", 022, source);
	if ( ferror ( out ) )
		return ( NOWRITE );

	if ( NOERR != ( result = encode(in, out) ) )
		return ( result );

	fprintf(out, "end\n");
	if ( ferror ( out ) )
		return ( NOWRITE );
	
	return ( NOERR );
}

int encode( FILE *in, FILE *out) {
	char buf[80];
	int i, n, result;

	for (;;) {
		/* 1 (up to) 45 character line */
		n = fr(in, buf, 45);
		if ( EOF == putc(ENC(n), out) )
			return ( NOWRITE );

		for (i=0; i<n; i += 3)
			if ( NOERR != ( result = eoutdec(&buf[i], out) ) )
				return ( result );

		if ( EOF == putc('\n', out) )
			return ( NOWRITE );
		if (n <= 0)
			break;
	}
	return ( NOERR );
}

int eoutdec ( char *p, FILE *f) {
	int c1, c2, c3, c4;

	c1 = *p >> 2;
	c2 = (*p << 4) & 060 | (p[1] >> 4) & 017;
	c3 = (p[1] << 2) & 074 | (p[2] >> 6) & 03;
	c4 = p[2] & 077;
	if ( EOF == putc(ENC(c1), f) )
		return ( NOWRITE );
	if ( EOF == putc(ENC(c2), f) )
		return ( NOWRITE );
	if ( EOF == putc(ENC(c3), f) )
		return ( NOWRITE );
	if ( EOF == putc(ENC(c4), f) )
		return ( NOWRITE );
	return ( NOERR );
}